"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SwipeDeck = SwipeDeck;
exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _SwipeDeck = require("../constants/SwipeDeck");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
var Direction;
(function (Direction) {
  Direction[Direction["Left"] = 0] = "Left";
  Direction[Direction["Right"] = 1] = "Right";
})(Direction || (Direction = {}));
//END TYPES

function SwipeDeck(_ref) {
  let {
    renderCard,
    data,
    onSwipeRight = () => {},
    onSwipeLeft = () => {},
    renderNoMoreCards = () => undefined,
    handleEndReached = () => {},
    swipeThreshold = _SwipeDeck.SWIPE_THRESHOLD,
    forceAnimationDuration = _SwipeDeck.FORCE_ANIMATION_DURATION,
    indentSideMultiplier = _SwipeDeck.INDENT_SIDE_MULTIPLIER,
    indentTopMultiplier = _SwipeDeck.INDENT_TOP_MULTIPLIER,
    initialRotation = _SwipeDeck.INITIAL_ROTATION,
    initialXPosition = _SwipeDeck.INITIAL_X_POSITION,
    initialYPosition = _SwipeDeck.INITIAL_Y_POSITION,
    rotationMultiplier = _SwipeDeck.ROTATION_MULTIPLIER,
    rotationRange = _SwipeDeck.ROTATION_RANGE
  } = _ref;
  const [cardIndex, setCardIndex] = (0, _react.useState)(0);
  const [dataChanged, setDataChanged] = (0, _react.useState)(false);
  const latestValue = (0, _react.useRef)(cardIndex);
  const position = (0, _react.useRef)(new _reactNative.Animated.ValueXY()).current;
  latestValue.current = cardIndex;
  (0, _react.useLayoutEffect)(() => {
    if (cardIndex !== 0 && cardIndex !== data.length - 1) {
      if (_reactNative.Platform.OS === 'android') _reactNative.UIManager.setLayoutAnimationEnabledExperimental && _reactNative.UIManager.setLayoutAnimationEnabledExperimental(true);
      _reactNative.LayoutAnimation.easeInEaseOut();
    }
    setDataChanged(false);
  }, [cardIndex, dataChanged]);
  (0, _react.useEffect)(() => {
    setDataChanged(true);
    setCardIndex(0);
  }, [data]);
  (0, _react.useEffect)(() => {
    cardIndex === data.length && handleEndReached();
  }, [data, cardIndex, handleEndReached]);
  const panResponder = (0, _react.useRef)(_reactNative.PanResponder.create({
    onStartShouldSetPanResponder: () => true,
    onPanResponderMove: (_event, gesture) => {
      position.setValue({
        x: gesture.dx,
        y: gesture.dy
      });
    },
    onPanResponderRelease: (_event, gesture) => {
      if (gesture.dx > swipeThreshold) {
        forceSwipe(Direction.Right);
      } else if (gesture.dx < -swipeThreshold) {
        forceSwipe(Direction.Left);
      } else {
        resetPosition();
      }
    }
  })).current;
  const forceSwipe = direction => {
    const x = direction === Direction.Right ? _SwipeDeck.SCREEN_WIDTH : -_SwipeDeck.SCREEN_WIDTH;
    _reactNative.Animated.timing(position, {
      toValue: {
        x: x,
        y: initialYPosition
      },
      duration: forceAnimationDuration,
      useNativeDriver: false
    }).start(() => onSwipeComplete(direction));
  };
  const onSwipeComplete = direction => {
    const item = data[latestValue.current];
    if (item) {
      direction === Direction.Right ? onSwipeRight(item) : onSwipeLeft(item);
    }
    position.setValue({
      x: initialXPosition,
      y: initialYPosition
    });
    setCardIndex(latestValue.current + 1);
  };
  const resetPosition = () => {
    _reactNative.Animated.spring(position, {
      toValue: {
        x: initialXPosition,
        y: initialYPosition
      },
      useNativeDriver: false
    }).start();
  };
  const getCardStyle = () => {
    const rotate = position.x.interpolate({
      inputRange: [-_SwipeDeck.SCREEN_WIDTH * rotationMultiplier, initialRotation, _SwipeDeck.SCREEN_WIDTH * rotationMultiplier],
      outputRange: [`-${rotationRange}deg`, `${initialRotation}deg`, `${rotationRange}deg`]
    });
    return {
      ...position.getLayout(),
      transform: [{
        rotate: rotate
      }]
    };
  };
  const renderCards = () => {
    if (cardIndex >= data.length) {
      return /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
        style: [getCardStyle(), styles.card]
      }, renderNoMoreCards());
    } else {
      return data.map((item, index) => {
        if (index === cardIndex) {
          return /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, _extends({
            key: item.id,
            style: [getCardStyle(), styles.card]
          }, panResponder.panHandlers), renderCard(item));
        } else if (index > cardIndex) {
          return /*#__PURE__*/_react.default.createElement(_reactNative.Animated.View, {
            style: [styles.card, {
              top: indentTopMultiplier * (index - cardIndex),
              left: indentSideMultiplier * (index - cardIndex)
            }],
            key: item.id
          }, renderCard(item));
        } else {
          return null;
        }
      }).reverse();
    }
  };
  return /*#__PURE__*/_react.default.createElement(_reactNative.View, null, renderCards());
}
const styles = _reactNative.StyleSheet.create({
  card: {
    opacity: 1,
    position: 'absolute',
    width: '100%'
  }
});
var _default = SwipeDeck;
exports.default = _default;
//# sourceMappingURL=SwipeDeck.js.map